/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.business.dao;

import edu.uoc.exemplefw.business.dto.CustomerDTO;

/**
 * Interface per a la implementaci del patr DAO per a la taula de clients.
 * Defineix el comportament de la classe que gestiona les dades dels clients 
 * a la base de dades.
 *
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public interface CustomerDAO {
	
	/**
	 * S'obtenen tots els clients de la taula de la base de dades
	 * 
	 * @return Llista de clients de la taula
	 */
	public CustomerDTO[] getAll();

	/**
	 * S'obtenen les dades del client segons el seu identificador
	 * 
	 * @param id Identificador del client
	 * @return
	 */
	public CustomerDTO getCustomerById(Integer id);

	/**
	 * S'obtenen les dades del client segons el seu NIF
	 * 
	 * @param id Identificador del client
	 * @return
	 */
	public CustomerDTO getCustomerByNIF(String nif);
	
	/**
	 * Inserta un client a la base de dades
	 * 
	 * @param customer Dades del client
	 */
	public void insertCustomer(CustomerDTO customer);

	/**
	 * Modifica les dades del client
	 * 
	 * @param customer Dades del client
	 */
	public void updateCustomer(CustomerDTO customer);

	/**
	 * Elimina el client de la base de dades
	 * 
	 * @param customer Dades del client
	 */
	public void deleteCustomer(CustomerDTO customer);
}