/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.business.dao.impl;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Map;
import java.util.Set;

import edu.uoc.exemplefw.business.dao.CustomerDAO;
import edu.uoc.exemplefw.business.dto.CustomerDTO;

/**
 * Classe que implementa el patr DAO per a la gesti de les dades del clients.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class CustomerDAOImpl implements CustomerDAO {

	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#getAll()
	 */
	public CustomerDTO[] getAll() {
		Map<Integer, CustomerDTO> customerData = BDD.customerTable;

		CustomerDTO[] result = new CustomerDTO[customerData.size()];

		Set<Integer> keys = customerData.keySet();
		int i = 0;
		for (Integer key : keys) {
			result[i] = customerData.get(key);
			i++;
		}
		
		// Ordena la llista per primerCognom, segonCognom i nom
		Arrays.sort(result, new Comparator<CustomerDTO>() {
			public int compare(CustomerDTO c1, CustomerDTO c2) {
				
				int compare = c1.getPrimerCognom().compareTo(c2.getPrimerCognom());
				if (compare == 0) {
					compare = c1.getSegonCognom().compareTo(c2.getSegonCognom());
					
					if (compare == 0) {
						compare = c1.getNom().compareTo(c2.getNom());
					}
				}
				return compare;
			}
		});
		
		return result;
	}

	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#getCustomerById(java.lang.Integer)
	 */
	public CustomerDTO getCustomerById(Integer id) {
		return BDD.customerTable.get(id);
	}
	
	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#getCustomerByNIF(java.lang.String)
	 */
	public CustomerDTO getCustomerByNIF(String nif) {
		
		Map<Integer, CustomerDTO> customerData = BDD.customerTable;
		
		Set<Integer> keys = customerData.keySet();
		
		for (Integer key : keys) {
			CustomerDTO customer = customerData.get(key);
			
			if (nif.equals(customer.getNif())) {
				return customer;
			}
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#updateCustomer(edu.uoc.exemplefw.business.dto.CustomerDTO)
	 */
	public void updateCustomer(CustomerDTO customer) {
		if (BDD.customerTable.containsKey(customer.getId())) {
			BDD.customerTable.put(customer.getId(), customer);
		}
	}

	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#insertCustomer(edu.uoc.exemplefw.business.dto.CustomerDTO)
	 */
	public void insertCustomer(CustomerDTO customer) {
		customer.setId(BDD.customerTable.size() + 1);
		BDD.customerTable.put(customer.getId(), customer);
	}

	/* (non-Javadoc)
	 * @see edu.uoc.exemplefw.business.dao.CustomerDAO#deleteCustomer(edu.uoc.exemplefw.business.dto.CustomerDTO)
	 */
	public void deleteCustomer(CustomerDTO customer) {
		BDD.customerTable.remove(customer.getId());
	}
}