/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.commands;

import java.text.ParseException;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.exemplefw.Constants;
import edu.uoc.exemplefw.business.dao.CustomerDAO;
import edu.uoc.exemplefw.business.dao.impl.CustomerDAOImpl;
import edu.uoc.exemplefw.business.dto.CustomerDTO;
import edu.uoc.exemplefw.forms.CustomerDetailForm;
import edu.uoc.exemplefw.utils.Utils;
import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.form.Form;

/**
 * Classe command per a la gesti del formulari per a les dades del client.
 * Mitjanant aquest formulari es pot donar d'alta, es pot modificar, 
 * consultar i donar de baixa clients de l'aplicaci.
 *
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class CustomerDetailCommand extends Command {

	static Log log = LogFactory.getLog(CustomerDetailCommand.class);

	// Constants de la classe
	private static final String ACCIO_ALTA				= "alta";
	private static final String ACCIO_MODIFICACIO 		= "modificar";
	private static final String ACCIO_BAIXA				= "baixa";
	
	private static final String BOTO_ACCEPTAR			= "acceptar";
	private static final String BOTO_CANCELAR			= "cancelar";
	
	private static final String RESULT_ERROR			= "error";
	private static final String RESULT_CORRECTE			= "success";
	private static final String RESULT_CUSTOMERDETAIL	= "customerDetailPage";

	public static final String ID_SELECCIONAT 			= "CustormerDetailCommand.idSeleccionat";
	public static final String ACCIO 					= "CustormerDetailCommand.accio";

	// DAO que utilitzar la classe	
	private CustomerDAO customerDAO = new CustomerDAOImpl();

	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#execute(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public String execute(ContextObject context, Form theForm) {
		
		CustomerDetailForm form = (CustomerDetailForm) theForm;
		
		// Es recuperen l'acci i el id del client seleccionat de la pgina llista de clients
		String accio = (String)context.getAttributeSession(ACCIO);
		Integer id = (Integer)context.getAttributeSession(ID_SELECCIONAT);
		
		// Si no s'ha indicat l'acci, s un error
		if (accio == null || "".equals(accio)) {
			log.error("No s'ha indicat cap valor al atribut " + ACCIO);
			return RESULT_ERROR;
		}
		form.setAccio(accio);
		
		// Si no hi ha operaci en aquest formulari
		if (form.getOperation() == null || "".equals(form.getOperation())) {
			
			// Si l'acci indicada a la llista de clients no s una alta, 
			// es recuperen les dades del client
			if (!ACCIO_ALTA.equals(accio)) {
				
				// Es necessita el identificador, si no hi s, s un error
				if (id == null) {
					log.error("No s'ha indicat cap valor al atribut " + ID_SELECCIONAT);
					return RESULT_ERROR;
				}
				CustomerDTO customer = customerDAO.getCustomerById(id);
		
				// Si no s'ha trobat el client, s un error
				if (customer == null) {
					log.error("No s'ha trobat el client amb el id indicat " + id);
					return RESULT_ERROR;
				}
				
				// Es mouen les dades del client al formulari de la pgina
				form.getCustomerDetail().setId(customer.getId());
				form.getCustomerDetail().setNom(customer.getNom());
				form.getCustomerDetail().setPrimerCognom(customer.getPrimerCognom());
				form.getCustomerDetail().setSegonCognom(customer.getSegonCognom());
				form.getCustomerDetail().setNif(customer.getNif());
				form.getCustomerDetail().setSexe(customer.getSexe());
				form.getCustomerDetail().setEmail(customer.getEmail());
				form.getCustomerDetail().setTelefon(customer.getTelefon());
				form.getCustomerDetail().setTipusComunicacio(customer.getTipusComunicacio());
				
				// Si la data de naixement est informada es mostra en el format apropiat
				if (customer.getDataNaixement() != null) {
					String data = Utils.dateToString(customer.getDataNaixement());
					form.getCustomerDetail().setDataNaixement(data);
				}
				
				// Si s una alta o una modificaci, el formulari estar en mode edici
				if (ACCIO_ALTA.equals(accio) || ACCIO_MODIFICACIO.equals(accio)) {
					form.setMode(Constants.ModeFormulari.MODE_EDICIO);
				} else {
					// Sino estar en mode consulta
					form.setMode(Constants.ModeFormulari.MODE_CONSULTA);
				}
			}
		} else {
			// Si l'operaci del formulari s "acceptar"
			if (BOTO_ACCEPTAR.equals(form.getOperation())) {
				CustomerDTO customer = new CustomerDTO();
				
				// S'indica el id seleccionat
				customer.setId(id);
				
				// Si l'acci s una alta o modificaci es mouen les dades del formulari
				// a l'objecte DTO del client
				if (ACCIO_ALTA.equals(accio) || ACCIO_MODIFICACIO.equals(accio)) {
					customer.setNom(form.getCustomerDetail().getNom());
					customer.setPrimerCognom(form.getCustomerDetail().getPrimerCognom());
					customer.setSegonCognom(form.getCustomerDetail().getSegonCognom());
					customer.setNif(form.getCustomerDetail().getNif());
					customer.setSexe(form.getCustomerDetail().getSexe());
					customer.setEmail(form.getCustomerDetail().getEmail());
					customer.setTelefon(form.getCustomerDetail().getTelefon());
					customer.setTipusComunicacio(form.getCustomerDetail().getTipusComunicacio());
					
					// Si la data de naixement est informada es converteix a Date
					Date data = null;
					if (!"".equals(form.getCustomerDetail().getDataNaixement())) {
						try {
							data = Utils.stringToDate(form.getCustomerDetail().getDataNaixement());
							
						} catch (ParseException e) {
							// Si s'ha produt un error, es mostrar la pgina d'error
							log.error("S'ha produt un error al donar format a la data de naixement " +
									form.getCustomerDetail().getDataNaixement());
							e.printStackTrace();
							return RESULT_ERROR;
						}
					}
					
					customer.setDataNaixement(data);
				}
				
				// Es crida al mtode del DAO apropiat segons l'acci a realitzar
				if (ACCIO_ALTA.equals(accio)) {
					customerDAO.insertCustomer(customer);
					
				} else if (ACCIO_MODIFICACIO.equals(accio)) {
					customerDAO.updateCustomer(customer);
					
				} else if (ACCIO_BAIXA.equals(accio)) {
					customerDAO.deleteCustomer(customer);
					
				}
				return RESULT_CORRECTE;
				
			// Si l'operaci s "cancellar" s correcte
			} else if (BOTO_CANCELAR.equals(form.getOperation())) {
				return RESULT_CORRECTE;
			}
		}

		// Per defecte es mostra la pgina amb el formulari de dades del client
		return RESULT_CUSTOMERDETAIL;
	}
	
	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#validate(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public boolean validate(ContextObject context, Form theForm) {

		CustomerDetailForm form = (CustomerDetailForm) theForm;
		
		boolean errors = false;
		
		// S'ob l'acci de la sessi
		String accio = (String)context.getAttributeSession(ACCIO);
		Integer id = (Integer)context.getAttributeSession(ID_SELECCIONAT);
		
		form.setAccio(accio);
		
		// Si l'acci es una alta o modificaci i l'operaci del formulari s el bot acceptar
		// es validen les dades del formulari
		if ((ACCIO_ALTA.equals(accio) || ACCIO_MODIFICACIO.equals(accio)) 
				&& BOTO_ACCEPTAR.equals(form.getOperation())) {
			
			// El N.I.F. s obligatori
			if ("".equals(form.getCustomerDetail().getNif())) {
				addError("customerDetail.error.nif.obligatori");
				errors = true;
			} else {
				// Si el N.I.F. est informat, es valida
				String nif = form.getCustomerDetail().getNif().toUpperCase();
				form.getCustomerDetail().setNif(nif);
				if (!Utils.validarNIF(nif)) {
					addError("customerDetail.error.nif.incorrecte");
					errors = true;
					
				} else {
					CustomerDTO customer = customerDAO.getCustomerByNIF(nif);
										
					if (ACCIO_ALTA.equals(accio) && customer != null) {
						// Si s una alta i el N.I.F. existeix, es un error
						addError("customerDetail.error.nif.duplicat");
						errors = true;
					} else {	
						if (ACCIO_MODIFICACIO.equals(accio) && !id.equals(customer.getId())) {
							// Si s una modificaci i el N.I.F. indicat pertany a altre client, es un error
							addError("customerDetail.error.nif.duplicat");
							errors = true;	
						}
					}
					
				}
			}
			
			// El nom s obligatori
			if ("".equals(form.getCustomerDetail().getNom())) {
				addError("customerDetail.error.nom.obligatori");
				errors = true;
			}
			
			// El primer cognoms obligatori
			if ("".equals(form.getCustomerDetail().getPrimerCognom())) {
				addError("customerDetail.error.primerCognom.obligatori");
				errors = true;
			}
			
			// El segon cognom s obligatori
			if ("".equals(form.getCustomerDetail().getSegonCognom())) {
				addError("customerDetail.error.segonCognom.obligatori");
				errors = true;			
			}
			
			// El sexe s obligatori
			if ("".equals(form.getCustomerDetail().getSexe())) {
				addError("customerDetail.error.sexe.obligatori");
				errors = true;
			}
			
			// La data de naixement s obligatoria
			if ("".equals(form.getCustomerDetail().getDataNaixement())) {
				addError("customerDetail.error.dataNaixement.obligatori");
				errors = true;
				
			} else {
				// Si la data est informada, es valida que sigui correcta
				try {
					Utils.stringToDate(form.getCustomerDetail().getDataNaixement());
					
				} catch (ParseException e) {
					// Si hi ha un error al format
					addError("customerDetail.error.dataNaixement.incorrecte");
					errors = true;
				}
			}
			// Si el tipus de comunicaci est informat
			if (form.getCustomerDetail().getTipusComunicacio() != null) {
				// Si s'ha seleccionat mtode de comunicaci telfon, aquest s obligatori
				if (form.getCustomerDetail().getHashTipusComunicacio().get(Constants.TipusComunicacio.COMUNICACIO_TELEFON) != null) {
					if ("".equals(form.getCustomerDetail().getTelefon())) {
						addError("customerDetail.error.telefon.obligatori");
						errors = true;
					}
				}
				
				// Si s'ha seleccionat mtode de comunicaci email, aquest s obligatori
				if (form.getCustomerDetail().getHashTipusComunicacio().get(Constants.TipusComunicacio.COMUNICACIO_EMAIL) != null) {
					if ("".equals(form.getCustomerDetail().getEmail())) {
						addError("customerDetail.error.email.obligatori");
						errors = true;
					}
				}
			}
			// Si el telfon est informat
			if (!"".equals(form.getCustomerDetail().getTelefon())) {
				// Validar el telfon
				if (!Utils.validarTelefon(form.getCustomerDetail().getTelefon())) {
					addError("customerDetail.error.telefon.incorrecte");
					errors = true;
				}
			}
			// Si l'adrea de correu electrnic est informada
			if (!"".equals(form.getCustomerDetail().getEmail())) {
				// Validar l'adrea de correu eletrnic
				if (!Utils.validarEmail(form.getCustomerDetail().getEmail())) {
					addError("customerDetail.error.email.incorrecte");
					errors = true;
				}
			}
		}

		return !errors;
	}
}