/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.commands;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.exemplefw.business.dao.CustomerDAO;
import edu.uoc.exemplefw.business.dao.impl.CustomerDAOImpl;
import edu.uoc.exemplefw.business.dto.CustomerDTO;
import edu.uoc.exemplefw.forms.CustomerListForm;
import edu.uoc.exemplefw.forms.CustomerListRow;
import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.form.Form;

/**
 * Classe command per a la gesti de la llista de clients de l'aplicaci
 *
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class CustomerListCommand extends Command {

	static Log log = LogFactory.getLog(CustomerListCommand.class);
	
	// Constants de la classe
	private static final String OPERATION_ALTA 			= "alta";
	private static final String RESULT_CUSTOMERLIST 	= "customerListPage";
	private static final String RESULT_CUSTOMERDETAIL	= "customerDetail";
	
	// DAO que utilitzar la classe	
	private CustomerDAO customerDAO = new CustomerDAOImpl();

	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#execute(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public String execute(ContextObject context, Form theForm) {

		// Per defecte es mostrar la pgina amb la llista de clients
		String result = RESULT_CUSTOMERLIST;
		
		CustomerListForm form = (CustomerListForm) theForm;

		// Si no s'ha indicat cap acci, es carrega la llista amb els clients de la base de dades
		if (form.getOperation() == null || "".equals(form.getOperation())) {
			
			// s'obt la llista de clients de la base de dades
			CustomerDTO[] customerList = customerDAO.getAll();

			form.setCustomerList(new ArrayList<CustomerListRow>());
			
			// Es passen les dades dels clients a objectes del formulari
			if (customerList != null && customerList.length > 0) {				
				for (CustomerDTO customerDTO : customerList) {

					CustomerListRow row = new CustomerListRow();

					row.setId(customerDTO.getId());
					row.setNif(customerDTO.getNif());
					row.setNom(customerDTO.getNom());
					row.setPrimerCognom(customerDTO.getPrimerCognom());
					row.setSegonCognom(customerDTO.getSegonCognom());

					form.getCustomerList().add(row);
				}
			}
			// S'inicialitza la selecci
			form.setIdSeleccionat("");
			
		} else {
			// Es guarda l'operaci a la sessi
			context.setAttributeSession(CustomerDetailCommand.ACCIO, form.getOperation());

			// Si la operaci no s una alta, es guarda a sessi el id del client seleccionat
			if (!OPERATION_ALTA.equals(form.getOperation())) {
				context.setAttributeSession(CustomerDetailCommand.ID_SELECCIONAT, Integer.valueOf(form.getIdSeleccionat()));
			}
			
			// Es navega cap al formlari amb les dades del client
			result = RESULT_CUSTOMERDETAIL;
		}
		// S'iniciatliza la operaci
		form.setOperation(null);
		
		return result;
	}

	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#validate(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public boolean validate(ContextObject context, Form theForm) {
		
		CustomerListForm form = (CustomerListForm) theForm;
		
		// Si s'ha informat l'operaci i aquesta no s una alta, es valida
		if (form.getOperation() != null && !OPERATION_ALTA.equals(form.getOperation())) {
			
			// Es valida que s'hagi seleccionat un client
			if (form.getIdSeleccionat() == null || "".equals(form.getIdSeleccionat()))  {
				addError("customerList.error.idCustomer.obligatori");
				form.setOperation(null);
				return false;
			}
		}
		return true;
	}
}