/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.commands;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.exemplefw.Constants;
import edu.uoc.exemplefw.business.dao.UserDAO;
import edu.uoc.exemplefw.business.dao.impl.UserDAOImpl;
import edu.uoc.exemplefw.business.dto.UserDTO;
import edu.uoc.exemplefw.forms.LoginForm;
import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.form.Form;

/**
 * Classe command per a la gesti del login dels usuaris a l'aplicaci
 *
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class LoginCommand extends Command {

	static Log log = LogFactory.getLog(LoginCommand.class);

	// Constants de la classe
	private static final String OPERATION_LOGIN = "login";	
	private static final String RESULT_SUCCESS 	= "success";
	private static final String RESULT_LOGIN 	= "login";
	
	// DAO que utilitzar la classe
	private UserDAO userDAO = new UserDAOImpl();
	
	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#execute(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public String execute(ContextObject context, Form theForm) {
		
		LoginForm form = (LoginForm)theForm;
		
		// Si la operaci indicada s la de login, s'obtindran les dades de 
		// l'usuari, es guardar a la sessi i es navegar cap a la segent 
		// pgina
		if (OPERATION_LOGIN.equals(form.getOperation())) {
			// S'obtenen les dades de l'usuari segons el nom d'usuari indicat
			UserDTO user = userDAO.getUserByUsername(form.getUsername());
			// Es guarden les dades de l'usuari a sessi
			context.setAttributeSession(Constants.Session.USER_SESSION, user);
			// Es navegar cap a la segent pgina
			return RESULT_SUCCESS;
		}
		
		// Es navegar cap a la pgina de login
		return RESULT_LOGIN;
	}
	
	/* (non-Javadoc)
	 * @see edu.uoc.fw.core.command.Command#validate(edu.uoc.fw.core.context.ContextObject, edu.uoc.fw.core.form.Form)
	 */
	@Override
	public boolean validate(ContextObject context, Form theForm) {
		
		boolean validate = true;
		
		LoginForm form = (LoginForm)theForm;		
		
		// Si l'operaci s un login es validen les dades introdudes
		if (OPERATION_LOGIN.equals(form.getOperation())) {
			// El nom d'usuari es obligatori
			if (form.getUsername() == null || "".equals(form.getUsername())) {
				addError("login.error.usuari.obligatori");
				validate = false;
			}
			// La contrasenya s obligatoria
			if (form.getPassword() == null || "".equals(form.getPassword())) {
				addError("login.error.password.obligatori");
				validate = false;
			}
			
			if (validate) {
				// Si tot s correcte, es valida que la contrasenya sigui de l'usuari 
				// indicat amb el nom d'usuari.
				if (!userDAO.isValidUser(form.getUsername(), form.getPassword())) {
					addError("login.error.usuari.incorrecte");
					validate = false;
				}
			}
		}
		
		return validate;
	}
}