/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.filter;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.uoc.exemplefw.Constants;
import edu.uoc.exemplefw.business.dto.UserDTO;

/**
 * Classe que filtra les peticions a l'aplicaci d'exemple.
 *  
 * Totes les peticions a l'aplicaci passen per aquest filtre, si la petici no s al command
 * de login, es comprova que l'usuari estigi "logat" (hagi fet login a l'aplicaci) si no s aix,
 * es redirigeix cap a la pgina de login.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class AccessControlFilter implements Filter {

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	@Override
	public void init(FilterConfig config) throws ServletException {
	}
	
	/* (non-Javadoc)
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
		
	 	HttpServletRequest req = (HttpServletRequest) request;
	    HttpServletResponse res = (HttpServletResponse) response;
	
	    // S'obt l'usuari guardat a la sessi, l'usuari logat
	    UserDTO user = (UserDTO) req.getSession().getAttribute(Constants.Session.USER_SESSION);
	    
	    // Si el command demanat s el de login, s'encadena la petici i es deixa passar
	    if (req.getServletPath().equals("/login.fw")) {
			chain.doFilter(request, response);
			
		// Si el command demanat no s el login i no hi ha usuari a la sessi o aquesta no s vlida
		// es fa un redirect cap a l'index de la aplicaci: no es deixa passar, es mostra la pgina de login
		} else if (user == null || !req.isRequestedSessionIdValid()) {
	    	res.sendRedirect("index.jsp");
	    	
	    } else {
	    	chain.doFilter(request, response);
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.servlet.Filter#destroy()
	 */
	@Override
	public void destroy() {
	}
}