/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.exemplefw.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Classe d'utilitats per a l'aplicaci d'exemple. 
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class Utils {

	/**
	 * Expressi regular per a validar l'adrea de correu electrnic
	 */
	private static final String PATTERN_EMAIL = "^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
            + "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";
	
	/**
	 * Expressi regular per a validar el N.I.F.
	 */
	private static final String PATTERN_NIF = "(\\d{1,8})([TRWAGMYFPDXBNJZSQVHLCKEtrwagmyfpdxbnjzsqvhlcke])";
	
	/**
	 * Expressi regular per a validar el nmero de telfon
	 */
	private static final String PATTER_PHONE = "(6|8|9)[0-9]{8}";
	
	/**
	 * Converteix un objecte <code>Date</code> en un objecte <code>String</code> en 
	 * el format <code>dd/MM/yyyy</code>
	 * 
	 * @param date data a convertir
	 * @return <code>String</code> amb la data en el format <code>dd/MM/yyyy</code>
	 */
	public static String dateToString(Date date) {
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		return sdf.format(date);
	}
	
	/**
	 * Mtode per a convertir un <code>String</code> en un objecte <code>Date</code>
	 * 
	 * @param string <code>String</code> amb la data a convertir en el format <code>dd/MM/yyyy</code>
	 * @return objecte Date amb la data indicada
	 * 
	 * @throws ParseException si hi ha errors de conversi
	 */
	public static Date stringToDate(String string) throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		sdf.setLenient(false); 
		return sdf.parse(string);
	}
	
	/**
	 * Es valida el N.I.F. indicat
	 * 
	 * @param nif N.I.F. a validar
	 * @return <code>true</code> si el N.I.F. s correcte, <code>false</code> en cas contrari 
	 */
	public static boolean validarNIF(String nif) {
	    boolean correcte = false;
	    Pattern pattern = Pattern.compile(PATTERN_NIF);
	    Matcher matcher = pattern.matcher(nif);
	    if (matcher.matches()) {
	        String lletra = matcher.group(2);
	        String lletres = "TRWAGMYFPDXBNJZSQVHLCKE";
	        int index = Integer.parseInt(matcher.group(1));
	        index = index % 23;
	        String reference = lletres.substring(index, index + 1);

	        if (reference.equalsIgnoreCase(lletra)) {
	            correcte = true;
	        } else {
	            correcte = false;
	        }
	    } else {
	        correcte = false;
	    }
	    return correcte;
	}
	
	/**
	 * Es validar el nmero de telfon indicat
	 * 
	 * @param telefon <code>String</code> amb el nmero de telfon a validar
	 * @return <code>true</code> si el nmero de telfon s correcte, <code>false</code> en cas contrari 
	 */
	public static boolean validarTelefon(String telefon) {

		Pattern pattern = Pattern.compile(PATTER_PHONE);
	    Matcher matcher = pattern.matcher(telefon);
	    
	    return matcher.matches();
	}
	
	/**
	 * Es validar l'adrea de correu electrnic indicada
	 * 
	 * @param email adrea de correu electrnic a validar
	 * @return <code>true</code> si l'adrea s correcte, <code>false</code> en cas contrari 
	 */
	public static boolean validarEmail(String email) {
		 
        Pattern pattern = Pattern.compile(PATTERN_EMAIL);
        Matcher matcher = pattern.matcher(email);
        
        return matcher.matches(); 
    }
}