/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.command;

import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.form.Form;

/**
 * Classe base per a les classes command de l'aplicaci. 
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public abstract class Command {

	/**
	 * Constant per a guardar el missatges del command a la petici.
	 */
	public static final String COMMAND_MESSAGES = "edu.uoc.fw.core.Command.COMMAND_MESSAGES";
	
	/**
	 * Constant per a accedir a la vista d'entrada configurada al command
	 */
	public static final String INPUT_VIEW = "edu.uoc.fw.core.Command.INPUT_VIEW";

	/**
	 * Missatges del command.
	 */
	private CommandMessages messages = new CommandMessages();


	protected void addError(String error) {
		this.messages.addError(error);
	}

	protected void addInfo(String info) {
		this.messages.addInfo(info);
	}
	
	/**
	 * Mtode per a l'execuci del command. 
	 * 
	 * @param context Context de la petici
	 * @param form Formulari associat al command
	 * @return
	 */
	public abstract String execute(ContextObject context, Form form);

	/**
	 * Mtode per a la validaci del formulari utilitzat pel command.
	 * 
	 * @param context Context de la petici
	 * @param form Formulari associat al command
	 * @return <code>true</code> si el formulari ha validat correctament, <code>false</code> si no ha validat.
	 */
	public boolean validate(ContextObject context, Form form) {
		return true;
	}

	public CommandMessages getMessages() {
		return this.messages;
	}
}