/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.config;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.exceptions.ConfigurationException;

/**
 * Configuraci del command indicada al fitxer xml de configuraci de l'aplicaci
 *
 * @author Luis Bravo Fuentes
 */
public class CommandConfig {

	/**
	 * Constants per a la configuraci de l'mbit del formulari que utilitzar el command
	 * 
	 * @author Luis Bravo Fuentes
	 * @version 1.0
	 */
	public static class Scope {
		public static final String SESSION = "session";
		public static final String REQUEST = "request";
	}
	/**
	 * Nom del command
	 */
	private String name;

	/**
	 * Classe que implementar el command
	 */
	private String type;

	/**
	 * Formulari que utilitzar el command
	 */
	private String form;

	/**
	 * mbit del formulari que utilitzar el command
	 */
	private String scope;
	
	/**
	 * Vista d'entrada per al command
	 */
	private String inputView;

	/**
	 * Hash amb els possibles resultats que es pot obtenir al executar el command
	 */
	private Map<String, ResultConfig> results = new HashMap<String, ResultConfig>();

	/**
	 * @return el valor de name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name el valor de name a establir
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return el valor de type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type el valor de type a establir
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return el valor de form
	 */
	public String getForm() {
		return form;
	}

	/**
	 * @param form el valor de form a establir
	 */
	public void setForm(String form) {
		this.form = form;
	}

	/**
	 * @return el valor de scope
	 */
	public String getScope() {
		return scope;
	}

	/**
	 * @param scope el valor de scope a establir
	 */
	public void setScope(String scope) {
		this.scope = scope;
	}
	
	/**
	 * @return el valor de inputView
	 */
	public String getInputView() {
		return inputView;
	}

	/**
	 * @param scope el valor de inputView a establir, a part es guarda com a possible resultat del command
	 * a la llista de results
	 */
	public void setInputView(String inputView) {
		this.inputView = inputView;
		
		// Se guarda la vista d'entrada com a resultat del command
		ResultConfig resultConfig = new ResultConfig(Command.INPUT_VIEW, inputView, false);
		
		results.put(Command.INPUT_VIEW, resultConfig);
	}
	
	/**
	 * Obt l'objecte <code>ResultConfig</code> segons el nom indicat
	 * 
	 * @param resultName nom del <code>ResultConfig</code>
	 * @return l'objecte <code>ResultConfig</code> si es troba, null en cas contrari
	 */
	public ResultConfig getResultByName(String resultName) {
		return results.get(resultName);
	}

	/**
	 * Afegeix un objecte <code>ResultConfig</code> a la llista de results
	 * 
	 * @param resultConfig objecte <code>ResultConfig</code> a afegir a la llista
	 */
	public void addResult(ResultConfig resultConfig) {
		results.put(resultConfig.getName(), resultConfig);
	}
	
	/**
	 * Es crea l'objecte command de la clase indicada a la configuraci
	 * 
	 * @return el nou objecte <code>Command</code> creat
	 * @throws ConfigurationException si es produeix un error a l'instanciar l'objecte
	 */
	public Command createCommand() throws ConfigurationException {

		Class<?> commandClass;
		Command command;
		try {
			commandClass = Class.forName(type);
			command = (Command) commandClass.newInstance();

		} catch (ClassNotFoundException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		} catch (IllegalAccessException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		} catch (InstantiationException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		}

		return command;
	}
	
	@Override
	public String toString() {
		String result = "CommandConfig: {name: \"" + name + "\", type: \"" + type + "\", form: \"" + form + "\", scope: \"" + scope + "\"";
		if (results != null && results.size() > 0) {
			result += ", results: {";

			Set<String> fordwards = results.keySet();
			for (String key : fordwards) {
				result += "\n{key: " + key + ", value: " + results.get(key) + "} ";
			}
			result += "} ";
		}
		return result;
	}
}