/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.config;

import edu.uoc.fw.core.exceptions.ConfigurationException;
import edu.uoc.fw.core.form.Form;

/**
 * Configuraci del formulari indicada al fitxer xml de configuraci de l'aplicaci
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class FormConfig {

	/**
	 * Nom del formulari
	 */
	private String name;

	/**
	 * Classe que implementa el formulari
	 */
	private String type;
	
	/**
	 * @return el valor de name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name el valor de name a establir
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return el valor de type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type el valor de type a establir
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Es crea l'objecte form de la clase indicada a la configuraci
	 * 
	 * @return
	 * @throws ConfigurationException
	 */
	public Form createForm() throws ConfigurationException {

		Class<?> formClass;
		Form form;
		try {
			formClass = Class.forName(type);
			form = (Form) formClass.newInstance();

		} catch (ClassNotFoundException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		} catch (IllegalAccessException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		} catch (InstantiationException e) {
			throw new ConfigurationException(e.getMessage() + ": " + type);

		}

		return form;
	}
	
	@Override
	public String toString() {
		return "FormConfig: {name: \"" + name + "\", type: \"" + type + "\"}";
	}
}