/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.config;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Classe per a gestionar la configuraci del multiidioma de l'aplicaci.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class ResourceBundleConfig {

	static Log log = LogFactory.getLog(Configuration.class);
	
	private static ResourceBundleConfig instance;
	
	/**
	 * Patr per a la carrega dels fitxers de properties.
	 */
	private String filePattern;
	
	/**
	 * Idioma per defecte per a l'aplicaci.
	 */
	private String defaultLanguage;
	
	/**
	 * Hash de recursos segons l'idioma
	 */
	private Map<String, ResourceBundle> resourceBundles = new HashMap<String, ResourceBundle>();
	
	/**
	 * @return el valor de filePattern
	 */
	public String getFilePattern() {
		return filePattern;
	}

	/**
	 * @param filePattern el valor de filePattern a establir
	 */
	public void setFilePattern(String filePattern) {
		this.filePattern = filePattern;
	}

	/**
	 * @return el valor de defaultLanguage
	 */
	public String getDefaultLanguage() {
		return defaultLanguage;
	}

	/**
	 * @param defaultLanguage el valor de defaultLanguage a establir
	 */
	public void setDefaultLanguage(String defaultLanguage) {
		this.defaultLanguage = defaultLanguage;
	}

	/**
	 * @return el valor de resourceBundles
	 */
	public Map<String, ResourceBundle> getResourceBundles() {
		return resourceBundles;
	}

	/**
	 * @param resourceBundles el valor de resourceBundles a establir
	 */
	public void setResourceBundles(Map<String, ResourceBundle> resourceBundles) {
		this.resourceBundles = resourceBundles;
	}

	private ResourceBundleConfig() {
	}
	
	/**
	 * S'afegeix un idioma segons el nom indicat a la hash de recursos
	 * 
	 * @param language
	 */
	protected void addLanguage(String language) {
		
		language = language.toLowerCase();
		
		// Es crea el locale segons el codi d'idioma indicat
		Locale locale = new Locale(language);
		
		// S'obt el fitxer de recursos segons el locale i el patr de fitxer
		ResourceBundle rb = ResourceBundle.getBundle(filePattern, locale);
		
		// S'afegeix a la hash
		resourceBundles.put(language, rb);
	}
	
	/**
	 * Es retorna la instancia de <code>ResourceBundleConfig</code> que s'utilitzar a l'aplicaci.
	 *  
	 * @return Instncia de la classe <code>ResourceBundleConfig</code> que cont la configuraci dels fitxers de propietats.
	 */
	public static synchronized ResourceBundleConfig getInstance() {

		if (instance == null) {
			instance = new ResourceBundleConfig();
		}
		return instance;
	}

	/**
	 * S'obt la propietat del fitxer de recursos de l'idioma indicat segons la clau indicada.
	 * 
	 * @param language Idioma del que es vol recuperar la clau.
	 * @param key Clau de la propietat al fitxer de recursos.
	 * @return Valor de la propietat segons l'idioma i la clau indicada.
	 */
	public String getMessage(String language, String key) {
		
		String value = key;
		
		// S'obte els recursos del idioma indicat
		ResourceBundle rs = resourceBundles.get(language);
		
		if (rs != null) {
			// Si el fitxer de recursos s'ha trobat, s'obt el literal indicat
			try {
				value = rs.getString(key);
			} catch (MissingResourceException e) {
				log.info("No s'ha trobat el valor per a l'idioma: " + language + " i la clau: " + key);
			}
		}
		return value;
	}

	@Override
	public String toString() {
		String str = "ResouceBundleConfig: {";
		str += "defaultLanguage: \"" + defaultLanguage + "\", ";
		str += "filePattern: \"" + filePattern + "\", ";
				
		Set<String> keys = resourceBundles.keySet();
		str += "languages: {";
		if (keys.size() > 0) {
			for (String key : keys) {
				str += "\n{key: \"" + key + "\", value: " + resourceBundles.get(key) + "} ";
			}
		}
		str += "} }\n";
		
		return str;
	}
}