/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.context;

import java.util.Enumeration;
import java.util.Map;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Classe que implementa el patr Context Object. 
 * Gestiona el context de la petici, encapsula les dades de la petici, la resposta i el servlet.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class ContextObject {

	private HttpServlet servlet;		
	private HttpServletRequest request;		
	private HttpServletResponse response;
	
	public ContextObject(HttpServlet servlet, HttpServletRequest request, HttpServletResponse response) {
		this.servlet = servlet;
		this.request = request;
		this.response = response;
	}

	/**
	 * @return el valor de servlet
	 */
	public HttpServlet getServlet() {
		return servlet;
	}

	/**
	 * @return el valor de request
	 */
	public HttpServletRequest getRequest() {
		return request;
	}

	/**
	 * @return el valor de response
	 */
	public HttpServletResponse getResponse() {
		return response;
	}

	/**
	 * Afegir un atribut a la petici
	 * 
	 * @param name nom de l'atribut
	 * @param value valor de l'atribut
	 */
	public void setAttributeRequest(String name, Object value) {
		request.setAttribute(name, value);
	}
	
	/**
	 * Obtenir el valor de l'atribut de l'mbit de la petici
	 * 
	 * @param name nom de l'atribut
	 * @return el valor de l'atribut indicat
	 */
	public Object getAttributeRequest(String name) {
		return request.getAttribute(name);
	}
	
	/**
	 * Eliminar l'atribut de l'mbit de la petici
	 * 
	 * @param name nom de l'atribut
	 */
	public void removeAttributeRequest(String name) {
		request.removeAttribute(name);
	}
	
	/**
	 * Afegir un atribut a la sessi
	 * 
	 * @param name nom de l'atribut
	 * @param value valor de l'atribut
	 */
	public void setAttributeSession(String name, Object value) {
		request.getSession().setAttribute(name, value);
	}
	
	/**
	 * Obtenir el valor de l'atribut de l'mbit de la sessi
	 * 
	 * @param name nom de l'atribut
	 * @return el valor de l'atribut indicat
	 */
	public Object getAttributeSession(String name) {
		return request.getSession().getAttribute(name);
	}

	/**
	 * Eliminar l'atribut de l'mbit de la sessi
	 * 
	 * @param name nom de l'atribut
	 */
	public void removeAttributeSession(String name) {
		request.getSession().removeAttribute(name);
	}
	
	/**
	 * Obtenir el nom de tots els atributs guardats a la sessi
	 * 
	 * @return <code>Enumeration</code> amb els noms dels atributs guardats en sessi
	 */
	@SuppressWarnings("unchecked")
	public Enumeration<String> getAttributeNamesSession() {
		return request.getSession().getAttributeNames();
	}
	
	/**
	 * Afegir un atribut a l'mbit d'aplicaci
	 * 
	 * @param name nom de l'atribut
	 * @param value valor de l'atribut
	 */
	public void setAttributeApplication(String name, Object value) {
		servlet.getServletContext().setAttribute(name, value);
	}

	/**
	 * Obtenir el valor de l'atribut de l'mbit de l'aplicaci
	 * 
	 * @param name nom de l'atribut
	 * @return el valor de l'atribut indicat
	 */
	public Object getAttributeApplication(String name) {
		return servlet.getServletContext().getAttribute(name);
	}

	/**
	 * Eliminar l'atribut de l'mbit de l'aplicaci
	 * 
	 * @param name nom de l'atribut
	 */
	public void removeAttributeApplication(String name) {
		servlet.getServletContext().removeAttribute(name);
	}
	
	/**
	 * Obtenir el nom de tots els atributs guardats a l'mbit d'aplicaci
	 * 
	 * @return <code>Enumeration</code> amb els noms dels atributs guardats a l'mbit d'aplicaci
	 */
	@SuppressWarnings("unchecked")
	public Enumeration<String> getAttributeNamesApplication() {
		return servlet.getServletContext().getAttributeNames();
	}
	
	/**
	 * Obt els noms i el seu valor dels parametres de la petici
	 * 
	 * @return taula de hash amb els noms dels parmetres de la petici i els seus valors
	 */
	@SuppressWarnings("unchecked")
	public Map<String, String> getParametersMap() {
		return request.getParameterMap();
	}
	
	/**
	 * Retorna la part de la URL de la petici que fa referencia al servlet
	 * 
	 * @return nom del servlet cridat
	 */
	public String getServletPath() {
		return request.getServletPath();
	}
}