/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.core.controller;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.fw.core.config.Configuration;
import edu.uoc.fw.core.config.ResultConfig;
import edu.uoc.fw.core.context.ContextObject;
import edu.uoc.fw.core.exceptions.ConfigurationException;

/**
 * Processa totes les peticions dirigides cap a l'aplicaci i les delega a l'ApplicationController
 * Implementaci del patr Front Controller.
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class FrontController extends HttpServlet {

	private static final long	serialVersionUID	= 4471177513534922155L;

	static Log log = LogFactory.getLog(FrontController.class);
	
	/**
	 * Es processa la petici. S'executa el command corresponent i es presenta la vista indicada a la configuraci
	 * 
	 * @param request
	 * @param response
	 * @throws ServletException
	 */
	protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException {

		ResultConfig resultConfig = null;
		try {
			// Crea l'objecte que gestiona el context de la petici
			ContextObject context = new ContextObject(this, request, response);
			
			// S'executa el command indicat en la petici i s'obt la segent vista a mostrar
			resultConfig = ApplicationController.execute(context);
			
		} catch (ConfigurationException e) {
			throw new ServletException("Error al processar la petici", e);
		}

		if (resultConfig == null) {
			throw new ServletException("Error, no s'ha obtingut cap resultat del command");
		}
		
		try {
			// Es presenta la vista indicada
			if (resultConfig.isRedirect()) {
				String goTo = resultConfig.getTo();
				// Si comena per "/" s'elimina aquest carcter
				if (goTo.startsWith("/")) {
					goTo = goTo.substring(1);
				}
				redirect(request, response, goTo);
			} else {
				fordward(request, response, resultConfig.getTo());
			}

		} catch (IOException e) {
			throw new ServletException("Error al tractar la resposta del command", e);
		}
	}

	/**
	 * Gestiona el mtode <code>GET</code> de la petici HTTP.
	 * 
	 * @param request
	 * @param response
	 */
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		// Es crida al mtode que processa la petici
		processRequest(request, response);
	}

	/**
	 * Gestiona el mtode <code>POST</code> de la petici HTTP.
	 * 
	 * @param request
	 * @param response
	 */
	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		// Es crida al mtode que processa la petici
		processRequest(request, response);
	}


	/**
	 * Presenta la vista indicada segons el nom de la pgina.
	 * 
	 * @param request la petici
	 * @param response la resposta
	 * @param goToResult el dest del forward
	 * 
	 * @throws ServletException
	 * @throws IOException
	 */
	private void fordward(HttpServletRequest request, HttpServletResponse response, String goToResult) throws ServletException, IOException {		
		RequestDispatcher dispatcher = this.getServletContext().getRequestDispatcher(response.encodeURL(goToResult));
		dispatcher.forward(request, response);
	}
	
	/**
	 * Fa una redirecci a l'adrea indicada
	 * 
	 * @param request la petici
	 * @param response la resposta
	 * @param goToResult el dest de la redirecci
	 * 
	 * @throws IOException
	 */
	private void redirect(HttpServletRequest request, HttpServletResponse response, String goToResult) throws IOException {
		String urlWithSessionID = response.encodeRedirectURL(goToResult);
		response.sendRedirect( urlWithSessionID );
	}

	/**
	 * Inicialitza el servlet FrontController
	 */
	@Override
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		
		try {
			// Inicialitza la configuraci del framework
			Configuration.initConfiguration(getServletContext(), config);
			
		} catch (ConfigurationException e) {
			log.error("Framework - FrontController - error al carregar la configuraci");
			throw new ServletException(e);
		}
		
		log.info("Framework - FrontController inicialitzat");
	}

	@Override
	public void destroy() {
	}
}