/*
 * Disseny i implementaci d'un marc de treball de presentaci per a aplicacions J2EE
 * 
 * Universitat Oberta de Catalunya (UOC)
 * 
 * Projecte de Final de Carrera - PFC - J2EE
 *
 * Alumne : Luis Bravo Fuentes
 * 
 * Consultor: Josep Maria Camps Riba
 */
package edu.uoc.fw.tags;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

import edu.uoc.fw.core.command.Command;
import edu.uoc.fw.core.command.CommandMessages;
import edu.uoc.fw.core.config.ResourceBundleConfig;

/**
 * Implementa l'etiqueta per a mostrar els missatges del command
 * 
 * @author Luis Bravo Fuentes
 * @version 1.0
 */
public class CommandMessagesTag extends SimpleTagSupport {

	/**
	 * Tipus dels missatges a mostrar ERROR o INFO.
	 * Si no s'indica cap tipus, es mostren tots dos.
	 */
	private String type;

	/**
	 * Classe css per al component
	 */
	private String cssClass;

	/**
	 * @return el valor de type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type el valor de type a establir
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return el valor de cssClass
	 */
	public String getCssClass() {
		return cssClass;
	}

	/**
	 * @param cssClass el valor de cssClass a establir
	 */
	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
	}

	private String printMessage(String cssClass, String message) {
		return "<li class=\"" + cssClass + "\">" + message + "</li>";
	}

	@Override
	public void doTag() throws JspException, IOException {

		PageContext pageContext = (PageContext) getJspContext();
		HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();

		// S'obtenen els missatges de la petici
		CommandMessages messages = (CommandMessages) request.getAttribute(Command.COMMAND_MESSAGES);
		
		if (messages != null && (messages.getErrorMessages().size() > 0 || messages.getInformationMessages().size() > 0)) {
			String htmlTag = "<div";
			if (cssClass != null) {
				htmlTag += " class=\"" + cssClass + "\"";
			}
			htmlTag += " /><ul>";
			
			ResourceBundleConfig resourceBundleConfig = ResourceBundleConfig.getInstance();
			
			String language = request.getLocale().getLanguage();
			
			if (getType() == null || "ERROR".equals(getType())) {
				for (String message : messages.getErrorMessages()) {
					String value = resourceBundleConfig.getMessage(language, message);
					htmlTag += printMessage("error", value);
				}
			}
			if (getType() == null || "INFO".equals(getType())) {
				for (String message : messages.getInformationMessages()) {
					String value = resourceBundleConfig.getMessage(language, message);
					htmlTag += printMessage("info", value);
				}
			}
			htmlTag += "</ul></div>";

			getJspContext().getOut().write(htmlTag);
		}
	}
}